@@ -29,6 +29,7 @@
 #include <QCoreApplication>
 #include <QtCore/QDir>
 #include <QProcess>
+#include <QDebug>
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -105,28 +106,33 @@ protected:
 
 KonfUpdate::KonfUpdate(QCommandLineParser *parser)
     : m_oldConfig1(Q_NULLPTR), m_oldConfig2(Q_NULLPTR), m_newConfig(Q_NULLPTR),
-      m_textStream(Q_NULLPTR), m_file(Q_NULLPTR)
+      m_bCopy(false), m_bOverwrite(false), m_textStream(Q_NULLPTR),
+      m_file(Q_NULLPTR), m_lineCount(-1)
 {
     bool updateAll = false;
 
-    m_config = new KConfig("kconf_updaterc");
+    m_config = new KConfig(QStringLiteral("kconf_updaterc"));
     KConfigGroup cg(m_config, QString());
 
     QStringList updateFiles;
 
-    m_debug = parser->isSet("debug");
+    m_debug = parser->isSet(QStringLiteral("debug"));
+
+    if (parser->isSet(QStringLiteral("testmode"))) {
+        QStandardPaths::setTestModeEnabled(true);
+    }
 
     m_bUseConfigInfo = false;
-    if (parser->isSet("check")) {
+    if (parser->isSet(QStringLiteral("check"))) {
         m_bUseConfigInfo = true;
-        const QString file = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kconf_update/" + parser->value("check"));
+        const QString file = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kconf_update/" + parser->value(QStringLiteral("check")));
         if (file.isEmpty()) {
-            qWarning("File '%s' not found.", parser->value("check").toLocal8Bit().data());
-            log() << "File '" << parser->value("check") << "' passed on command line not found" << endl;
+            qWarning("File '%s' not found.", parser->value(QStringLiteral("check")).toLocal8Bit().data());
+            log() << "File '" << parser->value(QStringLiteral("check")) << "' passed on command line not found" << endl;
             return;
         }
         updateFiles.append(file);
-    } else if (parser->positionalArguments().count()) {
+    } else if (!parser->positionalArguments().isEmpty()) {
         updateFiles += parser->positionalArguments();
     } else {
         if (cg.readEntry("autoUpdateDisabled", false)) {
@@ -160,9 +166,9 @@ KonfUpdate::~KonfUpdate()
     delete m_textStream;
 }
 
-QTextStream &operator<<(QTextStream &stream, const QStringList &lst)
+static QTextStream &operator<<(QTextStream &stream, const QStringList &lst)
 {
-    stream << lst.join(", ");
+    stream << lst.join(QStringLiteral(", "));
     return stream;
 }
 
@@ -200,7 +206,7 @@ QStringList KonfUpdate::findUpdateFiles(bool dirtyOnly)
 {
     QStringList result;
 
-    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "kconf_update", QStandardPaths::LocateDirectory);
+    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update"), QStandardPaths::LocateDirectory);
     Q_FOREACH (const QString &d, dirs) {
         const QDir dir(d);
 
@@ -239,15 +245,23 @@ bool KonfUpdate::checkFile(const QString &filename)
     int lineCount = 0;
     resetOptions();
     QString id;
+    bool foundVersion = false;
     while (!ts.atEnd()) {
         const QString line = ts.readLine().trimmed();
-        lineCount++;
+        if (line.startsWith(QLatin1String("Version=5"))) {
+            foundVersion = true;
+        }
+        ++lineCount;
         if (line.isEmpty() || (line[0] == '#')) {
             continue;
         }
-        if (line.startsWith("Id=")) {
+        if (line.startsWith(QLatin1String("Id="))) {
+            if (!foundVersion) {
+                qDebug() << QStringLiteral("Missing \"Version=5\", file \'%1\' will be skipped.").arg(filename);
+                return true;
+            }
             id = m_currentFilename + ':' + line.mid(3);
-        } else if (line.startsWith("File=")) {
+        } else if (line.startsWith(QLatin1String("File="))) {
             checkGotFile(line.mid(5), id);
         }
     }
@@ -315,13 +329,21 @@ bool KonfUpdate::updateFile(const QString &filename)
     ts.setCodec(QTextCodec::codecForName("ISO-8859-1"));
     m_lineCount = 0;
     resetOptions();
+    bool foundVersion = false;
     while (!ts.atEnd()) {
         m_line = ts.readLine().trimmed();
+        if (m_line.startsWith(QLatin1String("Version=5"))) {
+            foundVersion = true;
+        }
         m_lineCount++;
         if (m_line.isEmpty() || (m_line[0] == QLatin1Char('#'))) {
             continue;
         }
         if (m_line.startsWith(QLatin1String("Id="))) {
+            if (!foundVersion) {
+                qDebug() << QStringLiteral("Missing \"Version=5\", file \'%1\' will be skipped.").arg(filename);
+                break;
+            }
             gotId(m_line.mid(3));
         } else if (m_skip) {
             continue;
@@ -499,7 +521,7 @@ void KonfUpdate::gotFile(const QString &_file)
     if (!m_oldFile.isEmpty()) { // if File= is specified, it doesn't exist, is empty or contains only kconf_update's [$Version] group, skip
         if (m_oldConfig1 != Q_NULLPTR
                 && (m_oldConfig1->groupList().isEmpty()
-                    || (m_oldConfig1->groupList().count() == 1 && m_oldConfig1->groupList().first() == "$Version"))) {
+                    || (m_oldConfig1->groupList().count() == 1 && m_oldConfig1->groupList().at(0) == QLatin1String("$Version")))) {
             log() << m_currentFilename << ": File '" << m_oldFile << "' does not exist or empty, skipping" << endl;
             m_skipFile = true;
         }
@@ -760,6 +782,7 @@ void KonfUpdate::gotScript(const QString &_script)
         log() << m_currentFilename << ": Running script '" << script << "'" << endl;
     }
 
+    QStringList args;
     QString cmd;
     if (interpreter.isEmpty()) {
         cmd = path;
@@ -770,12 +793,12 @@ void KonfUpdate::gotScript(const QString &_script)
             m_skip = true;
             return;
         }
-        cmd = interpreterPath + ' ' + path;
+        cmd = interpreterPath;
+        args << path;
     }
 
     if (!m_arguments.isNull()) {
-        cmd += ' ';
-        cmd += m_arguments;
+        args += m_arguments;
     }
 
     QTemporaryFile scriptIn;
@@ -817,7 +840,7 @@ void KonfUpdate::gotScript(const QString &_script)
             log() << "Script contents is:" << endl << scriptFile.readAll() << endl;
         }
     }
-    proc.start(cmd);
+    proc.start(cmd, args);
     if (!proc.waitForFinished(60000)) {
         logFileError() << "update script did not terminate within 60 seconds: " << cmd << endl;
         m_skip = true;
@@ -923,15 +946,16 @@ void KonfUpdate::resetOptions()
 int main(int argc, char **argv)
 {
     QCoreApplication app(argc, argv);
-    app.setApplicationVersion("1.1");
+    app.setApplicationVersion(QStringLiteral("1.1"));
 
     QCommandLineParser parser;
     parser.addVersionOption();
     parser.setApplicationDescription(QCoreApplication::translate("main", "KDE Tool for updating user configuration files"));
     parser.addHelpOption();
-    parser.addOption(QCommandLineOption(QStringList() << "debug", QCoreApplication::translate("main", "Keep output results from scripts")));
-    parser.addOption(QCommandLineOption(QStringList() << "check", QCoreApplication::translate("main", "Check whether config file itself requires updating"), "update-file"));
-    //parser.addOption(QCommandLineOption(QStringList() << "+[file]", QCoreApplication::translate("main", "File to read update instructions from")));
+    parser.addOption(QCommandLineOption(QStringList() << QStringLiteral("debug"), QCoreApplication::translate("main", "Keep output results from scripts")));
+    parser.addOption(QCommandLineOption(QStringList() << QStringLiteral("testmode"), QCoreApplication::translate("main", "For unit tests only: use test directories to stay away from the user's real files")));
+    parser.addOption(QCommandLineOption(QStringList() << QStringLiteral("check"), QCoreApplication::translate("main", "Check whether config file itself requires updating"), QStringLiteral("update-file")));
+    parser.addPositionalArgument(QStringLiteral("files"), QCoreApplication::translate("main", "File(s) to read update instructions from"), QStringLiteral("[files...]"));
 
     // TODO aboutData.addAuthor(ki18n("Waldo Bastian"), KLocalizedString(), "bastian@kde.org");
 
